<?php
namespace Inti\Config;

use ArrayAccess;

/**
 * Configuration data.
 *
 * @package Inti\Config
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
interface ConfigInterface extends ArrayAccess
{
    /**
     * Returns true if any of the data changed during runtime.
     *
     * @return bool
     */
    public function isDirty(): bool;

    /**
     * Returns all the values different from the given $other.
     *
     * @param ConfigInterface $other
     * @return ConfigInterface
     */
    public function diff(ConfigInterface $other): ConfigInterface;

    /**
     * Returns a new config with values from the first overriden by $other.
     *
     * @param ConfigInterface $other
     * @return ConfigInterface
     */
    public function merge(ConfigInterface $other): ConfigInterface;

    /**
     * Returns the array copy of this object.
     *
     * @return array
     */
    public function getArrayCopy();

    /**
     * Replaces the contents of this array with $input.
     *
     * @param array $input
     * @return array
     */
    public function exchangeArray($input);
}
