<?php
namespace Inti\Config\Middleware;

use Inti\Config\Service\ConfigManagerInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

/**
 * Class ConfigMiddleware
 *
 * @package Inti\Config\Middleware
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
final class ConfigMiddleware implements MiddlewareInterface
{
    /** @var ConfigManagerInterface */
    private $config;

    /**
     * ConfigMiddleware constructor.
     *
     * @param ConfigManagerInterface $config
     */
    public function __construct(ConfigManagerInterface $config)
    {
        $this->config = $config;
    }


    /**
     * Process an incoming server request.
     *
     * Processes an incoming server request in order to produce a response.
     * If unable to produce the response itself, it may delegate to the provided
     * request handler to do so.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $response = $handler->handle(
            $request->withAttribute('config', $this->config->getConfig())
        );

        $this->config->flush();

        return $response;
    }
}
