<?php
namespace Inti\Config\Middleware;

use Inti\Config\Service\ConfigManager;
use Inti\Config\Service\ConfigManagerInterface;
use Psr\Container\ContainerInterface;
use Psr\Http\Server\MiddlewareInterface;
use RuntimeException;

/**
 * Class ConfigMiddlewareFactory
 *
 * @package Inti\Config\Middleware
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
final class ConfigMiddlewareFactory
{
    /**
     * @param ContainerInterface $container
     * @return MiddlewareInterface
     */
    public function __invoke(ContainerInterface $container): MiddlewareInterface
    {
        if (!$container->has(ConfigManager::CONTAINER_KEY)) {
            throw new RuntimeException(sprintf(
                "Configuration manager must be registered in the container under the key '%s'.",
                ConfigManager::CONTAINER_KEY
            ));
        }

        $config = $container->get(ConfigManager::CONTAINER_KEY);

        if (!$config instanceof ConfigManagerInterface) {
            throw new RuntimeException(sprintf(
                "Invalid configuration manager. Must be instance of %s, got %s instead.",
                ConfigManagerInterface::class,
                is_object($config) ? get_class($config) : gettype($config)
            ));
        }

        return new ConfigMiddleware($config);
    }
}
