<?php
namespace Inti\Config\Service;

use Inti\Config\ConfigInterface;
use Inti\Config\Writer\WriterInterface;
use Psr\Cache\CacheItemPoolInterface;
use Psr\Cache\InvalidArgumentException;

/**
 * Class CachedConfigManager
 *
 * @package Inti\Config\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
final class CachedConfigManager extends ConfigManager
{
    const CACHE_KEY = 'config';
    const CACHE_BASE_KEY = 'base';

    /** @var CacheItemPoolInterface */
    private $cache;

    /**
     * CachedConfigManager constructor.
     * @param CacheItemPoolInterface $cache
     * @param string|null $autoload
     * @param array $providers
     * @param WriterInterface|null $writer
     */
    public function __construct(
        CacheItemPoolInterface $cache,
        string $autoload = null,
        array $providers = [],
        WriterInterface $writer = null
    ) {
        parent::__construct($autoload, $providers, $writer);
        $this->cache = $cache;
    }

    /**
     * @throws InvalidArgumentException
     */
    public function flush(): void
    {
        $config = $this->getConfig();

        if ($config->isDirty()) {
            $this->cache->deleteItem(self::CACHE_KEY);
        }

        parent::flush();
    }

    /**
     * @return ConfigInterface
     * @throws InvalidArgumentException
     */
    protected function getBaseConfig(): ConfigInterface
    {
        if ($this->base === null) {
            $item = $this->cache->getItem(static::CACHE_BASE_KEY);

            if (!$item->isHit()) {
                $item->set(parent::getBaseConfig());
                $this->cache->save($item);
            }

            $this->base = $item->get();
        }

        return $this->base;
    }

    /**
     * @return ConfigInterface
     * @throws InvalidArgumentException
     */
    public function getConfig(): ConfigInterface
    {
        if ($this->config === null) {
            $item = $this->cache->getItem(static::CACHE_KEY);

            if (!$item->isHit()) {
                $item->set(parent::getConfig());
                $this->cache->save($item);
            }

            $this->config = $item->get();
        }

        return $this->config;
    }
}
