<?php
namespace Inti\Config\Service;

use Psr\Cache\CacheItemPoolInterface;

/**
 * Creates a new ConfigManager with the default configuration.
 *
 * @package Inti\Config\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
final class ConfigManagerFactory
{
    /**
     * Creates the manager.
     *
     * @param array $providers
     * @param CacheItemPoolInterface|null $cache
     * @return ConfigManagerInterface
     */
    public function __invoke(array $providers = [], CacheItemPoolInterface $cache = null): ConfigManagerInterface
    {
        $autoload = realpath('config/autoload');

        if ($cache) {
            return new CachedConfigManager($cache, is_dir($autoload) ? $autoload : null, $providers);
        }

        return new ConfigManager(is_dir($autoload) ? $autoload : null, $providers);
    }
}
