<?php
namespace Inti\Config\Writer;

use Inti\Config\ConfigInterface;

/**
 * Writes the configuration to *.local.php files.
 *
 * @package Inti\Config\Writer
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
final class LocalFileWriter implements WriterInterface
{
    /** @var string */
    private $dir;

    /**
     * LocalFileWriter constructor.
     *
     * @param string $dir
     */
    public function __construct(string $dir)
    {
        $this->dir = $dir;
    }

    public function filter($value)
    {
        if (is_array($value)) {
            return array_filter($value);
        }

        return $value;
    }

    /**
     * Writes the config value.
     *
     * @param string $key
     * @param ConfigInterface $config
     */
    public function write(string $key, ConfigInterface $config): void
    {
        $filename = sprintf('%s/%s.local.php', $this->dir, $key);
        $config = array_map([$this, 'filter'], $config->getArrayCopy());

        $local = [$key => $config];

        if (!empty($config)) {
            $contents = sprintf('<?php return %s;', var_export($local, true));
            file_put_contents($filename, $contents);
        } else if (file_exists($filename)) {
            unlink($filename);
        }
    }
}
