<?php
namespace IntiTest\Config\Middleware;

use Inti\Config\Config;
use Inti\Config\Middleware\ConfigMiddleware;
use Inti\Config\Service\ConfigManagerInterface;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\MockObject\RuntimeException;
use PHPUnit\Framework\TestCase;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Diactoros\ServerRequestFactory;

/**
 * Class ConfigMiddlewareTest
 *
 * @package IntiTest\Config\Middleware
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/warehouse
 */
final class ConfigMiddlewareTest extends TestCase
{
    /**
     * @throws RuntimeException
     */
    public function testConfigIsInjectedIntoRequestAndFlushIsCalled()
    {
        $config = new Config();
        /** @var ConfigManagerInterface|MockObject $manager */
        $manager = $this->createMock(ConfigManagerInterface::class);
        $manager->method('getConfig')->willReturn($config);
        $manager->expects($this->once())->method('flush');

        $middleware = new ConfigMiddleware($manager);
        $request = ServerRequestFactory::fromGlobals();

        $handler = $this->createMock(RequestHandlerInterface::class);
        /** @noinspection PhpParamsInspection */
        $handler->expects($this->once())->method('handle')->with(
            $this->callback(function (ServerRequestInterface $request) use ($config) {
                $this->assertSame($config, $request->getAttribute('config'));
                return true;
            })
        );

        $middleware->process($request, $handler);
    }
}
