<?php
namespace IntiTest\Config\Service;

use Inti\Config\ConfigInterface;
use Inti\Config\Service\ConfigManager;
use Inti\Config\Service\ConfigManagerInterface;
use PHPUnit\Framework\TestCase;

/**
 * Class ConfigManagerTest
 *
 * @package IntiTest\Config\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
final class ConfigManagerTest extends TestCase
{
    public function provider(): array
    {
        return [
            'from_provider' => [
                'provider_key' => 'provider value',
            ],
        ];
    }

    public function testCanReadTheConfig(): ConfigManagerInterface
    {
        /** @var ConfigManager|ConfigInterface[] $manager */
        $manager = new ConfigManager(__DIR__ . '/autoload', [
            [$this, 'provider'],
        ]);

        $this->assertEquals(['provider_key' => 'provider value'], $manager['from_provider']->getArrayCopy());
        $this->assertEquals(['key' => 'value', 'other' => 'other value'], $manager['test']->getArrayCopy());

        return $manager;
    }

    /**
     * @depends testCanReadTheConfig
     * @param ConfigManagerInterface $manager
     * @return ConfigManagerInterface
     */
    public function testCanFlushChanges(ConfigManagerInterface $manager): ConfigManagerInterface
    {
        $manager['test']['key'] = 'new value';
        $manager->flush();

        $filename = __DIR__ . '/autoload/test.local.php';
        $this->assertFileExists($filename);

        /** @noinspection PhpIncludeInspection */
        $local = include $filename;

        $this->assertEquals(['test' => ['key' => 'new value']], $local);

        return $manager;
    }

    /**
     * @depends testCanFlushChanges
     * @return ConfigManagerInterface
     */
    public function testCanReadLocalConfig(): ConfigManagerInterface
    {
        /** @var ConfigManager|ConfigInterface[] $manager */
        $manager = new ConfigManager(__DIR__ . '/autoload', [
            [$this, 'provider'],
        ]);

        $config = $manager->getConfig()->getArrayCopy();

        $this->assertCount(2, $config);
        $this->assertCount(2, $manager['test']);
        $this->assertEquals('new value', $manager['test']['key']);
        $this->assertEquals('other value', $manager['test']['other']);

        return $manager;
    }

    /**
     * @depends testCanReadLocalConfig
     * @param ConfigManagerInterface $manager
     */
    public function testWillRemoveEmptyFiles(ConfigManagerInterface $manager)
    {
        $manager['test']['key'] = 'value';
        $manager->flush();

        $filename = __DIR__ . '/autoload/test.local.php';
        $this->assertFileNotExists($filename);
    }
}
